import * as mysql from 'mysql';
import { Product } from 'ssn';
import IDatabaseRow from '../interfaces/IDatabaseRow';
import IProductData from '../interfaces/IProductData';
import writeLog from '../logger/writeLog';
import createStatement from './createStatement';
import * as database from './database';
import products from './products';

export async function init(): Promise<{ [key in Product]?: IProductData }> {
  const output: { [key in Product]?: IProductData } = {};

  await database.init();

  //Only create table if it does not exist yet
  const createIfNotExists = createStatement.replace(/^CREATE TABLE/, 'CREATE TABLE IF NOT EXISTS');
  await database.query(createIfNotExists);

  const { results: existingRows }: { results: IDatabaseRow[]} = await database.query('SELECT * FROM ssn_versions');

  existingRows.forEach(function(row: IDatabaseRow) {
    if (!products.includes(row.product)) {
      writeLog(`Product ${row.product} in database table is invalid.`);
      return;
    }

    output[row.product] = { http: row.http, manifest: row.manifest };
  });

  //If row does not exist, INSERT into table
  await Promise.all(products.filter((product) => output[product] === undefined).map(function(product) {
    output[product] = { http: -1, manifest: -1 };
    const queryTemplate = `INSERT INTO \`ssn_versions\` (\`product\`, \`manifest\`, \`http\`) VALUES (?, '-1', '-1');`;
    const query = mysql.format(queryTemplate, [String(product)]);
    return database.query(query);
  }));

  return output;
}

export function updateManifestVersion(product: Product, newVersion: number) {
  const queryTemplate = `UPDATE \`ssn_versions\` SET \`manifest\` = ? WHERE \`product\` = ?;`;
  const query = mysql.format(queryTemplate, [Number(newVersion), String(product)]);
  return database.query(query);
}
